# Generated by Django 5.2.5 on 2025-10-11 13:46

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("generic_notifications", "0003_remove_notification_notification_channels_gin_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="EmailFrequency",
            new_name="NotificationFrequency",
        ),
        migrations.AlterField(
            model_name="notificationfrequency",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="notification_frequencies",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
