from django.apps import AppConfig


class GenericNotificationsConfig(AppConfig):
    name = "generic_notifications"
    verbose_name = "Generic Notifications"
    default_auto_field = "django.db.models.BigAutoField"
