# Generated by Django 5.2.5 on 2025-10-11 12:28

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("generic_notifications", "0002_alter_disablednotificationtypechannel_id_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="notification",
            name="notification_channels_gin",
        ),
        migrations.RemoveIndex(
            model_name="notification",
            name="notification_unread_channel",
        ),
        migrations.RemoveIndex(
            model_name="notification",
            name="notification_recipient_channel",
        ),
        migrations.RemoveIndex(
            model_name="notification",
            name="notification_user_email_digest",
        ),
        migrations.RemoveField(
            model_name="notification",
            name="channels",
        ),
        migrations.RemoveField(
            model_name="notification",
            name="email_sent_at",
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(fields=["recipient", "read"], name="notification_unread_idx"),
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(fields=["recipient", "added"], name="notification_recipient_idx"),
        ),
    ]
