# Generated by Django 5.2.5 on 2025-10-20 09:35

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("generic_notifications", "0004_rename_emailfrequency_notificationfrequency"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="notificationfrequency",
            options={"verbose_name_plural": "Notification frequencies"},
        ),
        migrations.AlterField(
            model_name="notification",
            name="object_id",
            field=models.CharField(blank=True, max_length=36, null=True),
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(fields=["content_type", "object_id"], name="notification_target_idx"),
        ),
    ]
